@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2023 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2024 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -129,6 +129,7 @@ and concrete_syntax_term =
   | FunctionLiteral of concrete_syntax_funlit
   | Record of (string * concrete_syntax_term) list
   | Proj of (string * concrete_syntax_term)
+  | Let of (string * concrete_syntax_term) list * concrete_syntax_term
 
 
 (* Pretty printing of concrete terms *)
@@ -224,6 +225,9 @@ let rec print_concrete_term fmt ct =
       (Pp.print_list_delim ~start:Pp.lbrace ~stop:Pp.rbrace ~sep:Pp.semi print_field_value) fields_values
   | Proj (proj_name,proj_value) ->
     fprintf fmt "@[{%s =>@ %a}@]" proj_name print_concrete_term proj_value
+  | Let (ll, t) -> fprintf fmt "@[<hov 1>let %a@ in@ %a@]"
+                      (Pp.print_list_delim ~start:Pp.nothing ~stop:Pp.semi ~sep:Pp.semi (Pp.print_pair Pp.print_string print_concrete_term)) ll
+                      print_concrete_term t
 
 (* Helper functions for concrete terms *)
 
@@ -268,6 +272,7 @@ let rec subst_concrete_term subst t =
     )
   | Proj (proj_name,proj_value) ->
     Proj (proj_name, subst_concrete_term subst proj_value)
+  | Let (ll, t) -> Let (ll, subst_concrete_term subst t) (* Do I need to substitute inside the bound terms as well? *)
 let rec t_and_l_concrete = function
   | [] -> concrete_const_bool true
   | [f] -> f
@@ -385,6 +390,7 @@ let search_model_element_call_result model (call_id : Expr.expr_id option) =
 let cmp_attrs a1 a2 =
   String.compare a1.attr_string a2.attr_string
 
+(*
 let model_element_equal n1 n2 =
   let me_kind_equal k1 k2 = match k1,k2 with
   | Result, Result
@@ -403,8 +409,10 @@ let model_element_equal n1 n2 =
   Option.equal Loc.equal n1.me_location n2.me_location &&
   Sattr.equal n1.me_attrs n2.me_attrs &&
   Term.ls_equal n1.me_lsymbol n2.me_lsymbol
+*)
 
 (* FIXME: understand why some elements are duplicated *)
+(*
 let rec filter_duplicated l =
   let is_duplicated a l =
     List.exists (fun x -> model_element_equal a x) l in
@@ -412,6 +420,7 @@ let rec filter_duplicated l =
   | [] | [_] -> l
   | me :: l when is_duplicated me l -> filter_duplicated l
   | me :: l -> me :: filter_duplicated l
+*)
 
 let json_attrs attrs =
   let open Json_base in
@@ -671,6 +680,7 @@ let [@warning "-42"] rec json_of_concrete_term ct =
         "quant_t", json_of_concrete_term quant_t
       ]
     ]
+  | Let _ -> Record [ "type", String "Let"; "val", String "UNSUPPORTED" ]
   | Binop (op,t1,t2) ->
     let op_string = match op with
       | And -> "And"
@@ -761,7 +771,6 @@ let json_model_element me =
     ]
 
 let json_model_elements model_elements =
-  let model_elements = filter_duplicated model_elements in
   Json_base.List (List.map json_model_element model_elements)
 
 let json_model_elements_on_lines vc_term_loc (file_name, model_file) =
@@ -831,16 +840,13 @@ let print_model_element ?(print_locs=false) ~print_attrs fmt m_element =
         (Pp.print_option (Pretty.print_ty)) m_element.me_value.t_ty
         print_concrete_term m_element.me_concrete_value
 
-let print_model_elements ~filter_similar ~print_attrs ?(sep = Pp.newline)
-    fmt m_elements =
-  let m_elements =
-    if filter_similar then filter_duplicated m_elements else m_elements in
+let print_model_elements ~print_attrs ?(sep = Pp.newline) fmt m_elements =
   fprintf fmt "@[%a@]"
     (Pp.print_list sep
        (print_model_element ?print_locs:None ~print_attrs))
     m_elements
 
-let print_model_file ~filter_similar ~print_attrs fmt (filename, model_file) =
+let print_model_file ~print_attrs fmt (filename, model_file) =
   (* Relativize does not work on nighly bench: using basename instead. It
      hides the local paths. *)
   let filename = Filename.basename filename in
@@ -854,22 +860,15 @@ let print_model_file ~filter_similar ~print_attrs fmt (filename, model_file) =
       if n = 0 then Sattr.compare me1.me_attrs me2.me_attrs else n in
     let m_elements = List.sort cmp m_elements in
     fprintf fmt "  @[<v 2>Line %d:@ %a@]" line
-      (print_model_elements ~filter_similar ?sep:None ~print_attrs)
+      (print_model_elements ?sep:None ~print_attrs)
       m_elements in
   fprintf fmt "@[<v 0>File %s:@ %a@]" filename
     Pp.(print_list space pp) (Mint.bindings model_file)
 
-let print_model ~filter_similar ~print_attrs
-    fmt model =
-  Pp.print_list Pp.newline (print_model_file ~filter_similar ~print_attrs)
+let print_model ~print_attrs fmt model =
+  Pp.print_list Pp.newline (print_model_file ~print_attrs)
     fmt (Mstr.bindings model.model_files)
 
-let print_model_human ?(filter_similar = true) fmt model =
-  print_model ~filter_similar fmt model
-
-let print_model ?(filter_similar = true) ~print_attrs fmt model =
-  print_model ~filter_similar ~print_attrs fmt model
-
 let get_model_file model filename =
   Mstr.find_def empty_model_file filename model
 
@@ -918,7 +917,7 @@ let interleave_line ~filename:_ ~print_attrs start_comment end_comment
     let model_elements = Mint.find line_number model_file in
     let cntexmp_line =
       asprintf "@[<h 0>%s%s%a%s@]" (get_padding line) start_comment
-        (print_model_elements ~filter_similar:true ~sep:Pp.semi ~print_attrs)
+        (print_model_elements ~sep:Pp.semi ~print_attrs)
         model_elements end_comment in
     (* We need to know how many lines will be taken by the counterexample. This
        is ad hoc as we don't really know how the lines are split in IDE. *)
@@ -1187,6 +1186,7 @@ class clean = object (self)
     | Proj (s,v) -> self#proj s v
     | Function {args;body} -> self#func args body
     | FunctionLiteral {elts;others} -> self#funliteral elts others
+    | Let (ll, tt) -> self#lett ll tt
   method var v = Some (Var v)
   method const c = match c with
     | String v -> self#string v
@@ -1217,6 +1217,10 @@ class clean = object (self)
   method epsilon x t =
     Option.bind (self#value t) @@ fun t ->
     Some (Epsilon (x,t))
+  method lett ll tt =
+    Option.bind (self#value tt) @@ fun tt ->
+    let ll' = List.filter_map (fun (v,t) -> Option.bind (self#value t) @@ fun t -> Some (v, t)) ll in
+    Some (Let (ll',tt))
   method quant q vars t =
     Option.bind (self#value t) @@ fun t ->
     Some (Quant (q,vars,t))
@@ -1261,13 +1265,12 @@ type model_parser = printing_info -> string -> model
 type raw_model_parser = printing_info -> string -> model_element list
 
 let debug_elements elts =
-  let print_elements = print_model_elements ~sep:Pp.semi ~print_attrs:true
-    ~filter_similar:false in
+  let print_elements = print_model_elements ~sep:Pp.semi ~print_attrs:true in
   Debug.dprintf debug "@[<v>Elements:@ %a@]@." print_elements elts;
   elts
 
 let debug_files files =
-  let print_file = print_model_file ~filter_similar:false ~print_attrs:true in
+  let print_file = print_model_file ~print_attrs:true in
    Debug.dprintf debug "@[<v>Files:@ %a@]@."
      (Pp.print_list Pp.newline print_file) (Mstr.bindings files);
    files
