@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2023 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2024 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -31,7 +31,7 @@ let meta_get_counterexmp =
 
 let get_counterexmp task =
   let ce_meta = Task.find_meta_tds task meta_get_counterexmp in
-  not (Theory.Stdecl.is_empty ce_meta.tds_set)
+  not (Task.HStdecl.is_empty ce_meta)
 
 (** drivers *)
 
@@ -322,8 +322,8 @@ let update_task = let ht = Hint.create 5 in fun drv ->
     let update task0 =
       (* add requested theorie *)
       let task0 = Mid.fold (fun _ (th,th') task ->
-          let tdcs = (Task.find_clone_tds task0 th).tds_set in
-          Stdecl.fold (fun tdc task -> match tdc.td_node with
+          let tdcs = Task.find_clone_tds task0 th in
+          HStdecl.fold (fun tdc task -> match tdc.td_node with
               | Use _ -> Task.use_export task th'
               | Clone (_,_) ->
                   (* We do nothing in case of clone *)
@@ -334,8 +334,8 @@ let update_task = let ht = Hint.create 5 in fun drv ->
       in
       (* apply metas *)
       let task0 = Mid.fold (fun _ (th,tdms) task ->
-          let tdcs = (Task.find_clone_tds task0 th).tds_set in
-          Stdecl.fold (fun tdc task ->
+          let tdcs = Task.find_clone_tds task0 th in
+          HStdecl.fold (fun tdc task ->
               Stdecl.fold (fun tdm task -> match tdc.td_node with
                   | Use _ -> add_tdecl task tdm
                   | Clone (_,sm) ->
@@ -444,7 +444,7 @@ let file_name_of_task ?old ?inplace ?interactive drv task =
           ~goal_name:pr.pr_name.id_string
 
 let prove_task_prepared
-      ~command ~config ~limit ?old ?inplace ?interactive drv task =
+      ~command ~config ~limits ?old ?inplace ?interactive drv task =
   let buf = Buffer.create 1024 in
   let fmt = formatter_of_buffer buf in
   let old_channel = Option.map open_in old in
@@ -456,7 +456,7 @@ let prove_task_prepared
   let get_model = if get_counterexmp task then Some printing_info else None in
   let res =
     Call_provers.call_on_buffer
-      ~command ~config ~limit ~gen_new_file ?inplace ~filename
+      ~command ~config ~limits ~gen_new_file ?inplace ~filename
       ~res_parser:drv.drv_res_parser
       ~get_model buf
   in
@@ -464,7 +464,7 @@ let prove_task_prepared
   res
 
 let prove_buffer_prepared
-    ~command ~config ~limit
+    ~command ~config ~limits
     ?(input_file="f")
     ?(theory_name="T")
     ?(goal_name="vc")
@@ -472,16 +472,16 @@ let prove_buffer_prepared
     drv buffer =
   let filename = get_filename drv ~input_file ~theory_name ~goal_name in
   Call_provers.call_on_buffer
-    ~command ~config ~limit
+    ~command ~config ~limits
     ~gen_new_file:true ~filename
     ~res_parser:drv.drv_res_parser
     ~get_model buffer
 
 let prove_task
-      ~command ~config ~limit ?old ?inplace ?interactive drv task =
+      ~command ~config ~limits ?old ?inplace ?interactive drv task =
   let task = prepare_task drv task in
   prove_task_prepared
-    ~command ~config ~limit ?interactive ?old ?inplace drv task
+    ~command ~config ~limits ?interactive ?old ?inplace drv task
 
 (* exception report *)
 
