@@ -1,7 +1,7 @@
 (********************************************************************)
 (*                                                                  *)
 (*  The Why3 Verification Platform   /   The Why3 Development Team  *)
-(*  Copyright 2010-2023 --  Inria - CNRS - Paris-Saclay University  *)
+(*  Copyright 2010-2024 --  Inria - CNRS - Paris-Saclay University  *)
 (*                                                                  *)
 (*  This software is distributed under the terms of the GNU Lesser  *)
 (*  General Public License version 2.1, with the special exception  *)
@@ -318,6 +318,14 @@ let mk_decl_meta = let r = ref 0 in fun meta node pure ->
 
 let mk_decl = mk_decl_meta []
 
+let meta_depends =
+  Theory.(
+    register_meta
+      ~desc:
+        "declares an dependency between a proposition and a logic symbol. Used \
+         by the transformation `remove_unused`"
+      "remove_unused:dependency" [ MTprsymbol; MTlsymbol ])
+
 let axiom_of_invariant itd =
   let ts = itd.itd_its.its_ts in
   let inv = t_and_simp_l itd.itd_invariant in
@@ -331,6 +339,44 @@ let axiom_of_invariant itd =
   let trl = Mvs.fold (fun _ t trl -> [t] :: trl) sbs [] in
   t_forall_close [u] trl (t_subst sbs inv)
 
+let injectivity ty id pjs =
+  let eq_id =
+    let attrs = suffix_attr_name ~attrs:id.id_attrs "'eq" in
+    id_fresh ~attrs ?loc:id.id_loc (id.id_string ^ "'eq") in
+  let pr_id =
+    let attrs = suffix_attr_name ~attrs:id.id_attrs "'inj" in
+    let attrs = Sattr.add Theory.attr_w_non_conservative_extension_no attrs in
+    id_fresh ~attrs ?loc:id.id_loc (id.id_string ^ "'inj") in
+  let a = create_vsymbol (id_fresh "a") ty in
+  let b = create_vsymbol (id_fresh "b") ty in
+    let xs = [a;b] in
+  let ta = t_var a in
+  let tb = t_var b in
+  let eq = create_psymbol eq_id [ty;ty] in
+  let pr = create_prsymbol pr_id in
+  let eq_fields =
+      let lta = [ta] in
+      let ltb = [tb] in
+      List.map
+        (fun pj ->
+           let r = pj.ls_value in
+           t_equ (t_app pj lta r) (t_app pj ltb r)
+        ) pjs in
+  let eq_axiom =
+    t_forall_close xs [] @@
+    t_implies (t_app eq [ta;tb] None) (t_equ ta tb) in
+  [
+    create_logic_decl [make_ls_defn eq xs (t_and_l eq_fields)] ;
+    create_prop_decl Paxiom pr eq_axiom ;
+  ],
+  [(meta_depends,[Theory.MApr pr; Theory.MAls eq])]
+
+let record_injectivity id itd =
+  if itd.itd_its.its_private then [],[] else
+    let ts = itd.itd_its.its_ts in
+    let ty = ty_app ts (List.map ty_var ts.ts_args) in
+    injectivity ty id (List.map ls_of_rs itd.itd_fields)
+
 let create_type_decl dl =
   if dl = [] then invalid_arg "Pdecl.create_type_decl";
   let conv_itd ({itd_its = s} as itd) =
@@ -342,10 +388,11 @@ let create_type_decl dl =
         let pj_id =
           let attrs = suffix_attr_name ~attrs:id.id_attrs "'int" in
           id_fresh ~attrs ?loc:id.id_loc (nm ^ "'int") in
-        let pj_ls = create_fsymbol pj_id [ty_app ts []] ty_int in
+        let ty = ty_app ts [] in
+        let pj_ls = create_fsymbol pj_id [ty] ty_int in
         let pj_decl = create_param_decl pj_ls in
-        (* Create the projection meta for pj_decl *)
-        let meta_proj_pj = (Theory.meta_projection, [Theory.MAls pj_ls]) in
+        (* Create the model projection meta for pj_decl *)
+        let meta_proj_pj = (Theory.meta_model_projection, [Theory.MAls pj_ls]) in
         (* create max attribute *)
         let max_id =
           let attrs = suffix_attr_name ~attrs:id.id_attrs "'maxInt" in
@@ -362,17 +409,21 @@ let create_type_decl dl =
         let const = Constant.int_const ir.Number.ir_lower in
         let min_defn = t_const const ty_int in
         let min_decl = create_logic_decl [make_ls_defn min_ls [] min_defn] in
-        let pure = [create_ty_decl ts; pj_decl; max_decl; min_decl] in
+        (* injectivity of projection *)
+        let inj,meta_inj = injectivity ty id [pj_ls] in
+        (* build *)
+        let pure = create_ty_decl ts :: pj_decl :: max_decl :: min_decl :: inj in
         let meta = Theory.(meta_range, [MAts ts; MAls pj_ls]) in
-        mk_decl_meta [meta; meta_proj_pj] (PDtype [itd]) pure
+        mk_decl_meta (meta :: meta_proj_pj :: meta_inj) (PDtype [itd]) pure
     | _, _, Float ff ->
         let pj_id =
           let attrs = suffix_attr_name ~attrs:id.id_attrs "'real" in
           id_fresh ~attrs ?loc:id.id_loc (nm ^ "'real") in
-        let pj_ls = create_fsymbol pj_id [ty_app ts []] ty_real in
+        let ty = ty_app ts [] in
+        let pj_ls = create_fsymbol pj_id [ty] ty_real in
         let pj_decl = create_param_decl pj_ls in
-        (* Create the projection meta for pj_decl *)
-        let meta_proj_pj = (Theory.meta_projection, [Theory.MAls pj_ls]) in
+        (* Create the model projection meta for pj_decl *)
+        let meta_proj_pj = (Theory.meta_model_projection, [Theory.MAls pj_ls]) in
         (* create finiteness predicate *)
         let iF_id =
           let attrs = suffix_attr_name ~attrs:id.id_attrs "'isFinite" in
@@ -393,18 +444,22 @@ let create_type_decl dl =
         let sb_ls = create_fsymbol sb_id [] ty_int  in
         let sb_defn = t_nat_const ff.Number.fp_significand_digits in
         let sb_decl = create_logic_decl [make_ls_defn sb_ls [] sb_defn] in
-        let pure = [create_ty_decl ts; pj_decl; iF_decl; eb_decl; sb_decl] in
+        (* build *)
+        let pure = [create_ty_decl ts;pj_decl;iF_decl;eb_decl;sb_decl] in
         let meta = Theory.(meta_float, [MAts ts; MAls pj_ls; MAls iF_ls]) in
         mk_decl_meta [meta; meta_proj_pj] (PDtype [itd]) pure
     | fl, _, NoDef when itd.itd_invariant <> [] ->
-        let inv = axiom_of_invariant itd in
-        let pr =
+        (* type invariant *)
+        let pinv =
           let attrs = suffix_attr_name ~attrs:id.id_attrs "'invariant" in
           create_prsymbol (id_fresh ~attrs ?loc:id.id_loc (nm ^ "'invariant")) in
-        let ax = create_prop_decl Paxiom pr inv in
+        let inv = create_prop_decl Paxiom pinv (axiom_of_invariant itd) in
+        (* injectivity *)
+        let inj,meta_inj = record_injectivity id itd in
+        (* build *)
         let add_fd s dl = create_param_decl (ls_of_rs s) :: dl in
-        let pure = create_ty_decl ts :: List.fold_right add_fd fl [ax] in
-        mk_decl (PDtype [itd]) pure
+        let pure = create_ty_decl ts :: List.fold_right add_fd fl (inv::inj) in
+        mk_decl_meta meta_inj (PDtype [itd]) pure
     | fl, [], NoDef ->
         let add_fd s dl = create_param_decl (ls_of_rs s) :: dl in
         let pure = create_ty_decl ts :: List.fold_right add_fd fl [] in
@@ -482,8 +537,8 @@ let create_let_decl ld =
                 t_subst_fmla v t f in
     List.map conv ql in
   let attrs = sattr_w_nce_no in
-  let cty_axiom id cty f axms =
-    if t_equal f t_true then axms else
+  let cty_axiom lsopt id cty f axms metas =
+    if t_equal f t_true then axms, metas else
     (* we do not care about aliases for pure symbols *)
     let add_old o v m = Mvs.add o.pv_vs (t_var v.pv_vs) m in
     let sbs = Mpv.fold add_old cty.cty_oldies Mvs.empty in
@@ -491,38 +546,44 @@ let create_let_decl ld =
     let args = List.map (fun v -> v.pv_vs) cty.cty_args in
     let f = t_forall_close_simp args [] f in
     let f = t_forall_close (Mvs.keys (t_vars f)) [] f in
-    create_prop_decl Paxiom (create_prsymbol id) f :: axms in
-  let post_axioms id nm cty t axms =
+    let pr = create_prsymbol id in
+    let metas = match lsopt with
+      | None -> metas
+      | Some ls -> Theory.(meta_depends,[MApr pr; MAls ls]) :: metas
+    in
+    create_prop_decl Paxiom pr f :: axms, metas
+  in
+  let post_axioms lsopt id nm cty t axms metas =
     let cnt = ref (-1) in
-    let fold f axms =
+    let fold f (axms,metas) =
       let nm =
         if !cnt = -1 then nm else
         Printf.sprintf "%s'%d" nm !cnt
       in
       incr cnt;
-      cty_axiom (id_derive ~attrs nm id) cty f axms
+      cty_axiom lsopt (id_derive ~attrs nm id) cty f axms metas
     in
-    List.fold_right fold (conv_post t cty.cty_post) axms
+    List.fold_right fold (conv_post t cty.cty_post) (axms,metas)
   in
-  let add_rs sm s ({c_cty = cty} as c) (abst,defn,axms) =
+  let add_rs sm s ({c_cty = cty} as c) (abst,defn,axms,metas) =
     match s.rs_logic with
     | RLpv _ -> invalid_arg "Pdecl.create_let_decl"
-    | RLnone -> abst, defn, axms
+    | RLnone -> abst, defn, axms, metas
     | RLlemma ->
-        let axms = if ity_equal cty.cty_result ity_unit then
-          post_axioms s.rs_name s.rs_name.id_string cty t_void axms
+        let axms, metas = if ity_equal cty.cty_result ity_unit then
+          post_axioms None s.rs_name s.rs_name.id_string cty t_void axms metas
         else match cty.cty_post with
           | q::ql ->
               let v, f = open_post q in
               let fl = f :: conv_post (t_var v) ql in
               let f = t_exists_close [v] [] (t_and_simp_l fl) in
-              cty_axiom (id_clone ~attrs s.rs_name) cty f axms
-          | [] -> axms in
-        abst, defn, axms
+              cty_axiom None (id_clone ~attrs s.rs_name) cty f axms metas
+          | [] -> (axms,metas) in
+        abst, defn, axms, metas
     | RLls ({ls_name = id} as ls) ->
         let vl = List.map (fun v -> v.pv_vs) cty.cty_args in
         let hd = t_app ls (List.map t_var vl) ls.ls_value in
-        let axms = post_axioms id (id.id_string ^ "'spec") cty hd axms in
+        let axms, metas = post_axioms (Some ls) id (id.id_string ^ "'spec") cty hd axms metas in
         let c = if Mrs.is_empty sm then c else c_rs_subst sm c in
         begin match c.c_node with
         | Cany | Capp _ | Cpur _ ->
@@ -532,38 +593,38 @@ let create_let_decl ld =
                a definition from it. We only produce definitions via
                term_of_expr from a Cfun, and the user must eta-expand
                to obtain one. *)
-            create_param_decl ls :: abst, defn, axms
+            create_param_decl ls :: abst, defn, axms, metas
         | Cfun e ->
             begin match term_of_expr ~prop:(ls.ls_value = None) e with
             | Some f when cty.cty_pre = [] ->
-                abst, (ls, vl, f) :: defn, axms
+                abst, (ls, vl, f) :: defn, axms, metas
             | Some f ->
                 let f = t_insert hd f and nm = id.id_string ^ "'def" in
-                let axms = cty_axiom (id_derive ~attrs nm id) cty f axms in
-                create_param_decl ls :: abst, defn, axms
+                let axms, metas = cty_axiom (Some ls) (id_derive ~attrs nm id) cty f axms metas in
+                create_param_decl ls :: abst, defn, axms, metas
             | None when cty.cty_post = [] ->
-                let axms = match post_of_expr hd e with
+                let axms, metas = match post_of_expr hd e with
                   | Some f ->
                       let nm = id.id_string ^ "'def" in
-                      cty_axiom (id_derive ~attrs nm id) cty f axms
-                  | None -> axms in
-                create_param_decl ls :: abst, defn, axms
+                      cty_axiom (Some ls) (id_derive ~attrs nm id) cty f axms metas
+                  | None -> axms, metas in
+                create_param_decl ls :: abst, defn, axms, metas
             | None ->
-                create_param_decl ls :: abst, defn, axms
+                create_param_decl ls :: abst, defn, axms, metas
             end
         end in
-  let abst, defn, axms = match ld with
+  let abst, defn, axms, metas = match ld with
     | LDvar ({pv_vs = {vs_name = {id_loc = loc}}},e) ->
         if not (ity_closed e.e_ity) then
           Loc.errorm ?loc "Top-level variables must have monomorphic type";
-        [], [], []
+        [], [], [], []
     | LDrec rdl ->
         let add_rd sm d = Mrs.add d.rec_rsym d.rec_sym sm in
         let sm = List.fold_left add_rd Mrs.empty rdl in
         let add_rd d dl = add_rs sm d.rec_sym d.rec_fun dl in
-        List.fold_right add_rd rdl ([],[],[])
+        List.fold_right add_rd rdl ([],[],[],[])
     | LDsym (s,c) ->
-        add_rs Mrs.empty s c ([],[],[]) in
+        add_rs Mrs.empty s c ([],[],[],[]) in
   let defn = if defn = [] then [] else
     let dl = List.map (fun (s,vl,t) -> make_ls_defn s vl t) defn in
     try [create_logic_decl dl] with Decl.NoTerminationProof _ ->
@@ -575,7 +636,7 @@ let create_let_decl ld =
       let ax = t_forall_close vl [] (t_insert hd t) in
       create_prop_decl Paxiom pr ax in
     abst @ List.map mk_ax defn in
-  mk_decl (PDlet ld) (abst @ defn @ axms)
+  mk_decl_meta metas (PDlet ld) (abst @ defn @ axms)
 
 let create_exn_decl xs =
   if not (ity_closed xs.xs_ity) then Loc.errorm ?loc:xs.xs_name.id_loc
